from rasax.community.config import team_name
from rasax.community.services.projects_service import ProjectsService
from rasax.community.services.settings_service import SettingsService


def test_storing_of_domain(projects_service: ProjectsService,
                           settings_service: SettingsService):
    test_project = "project"
    settings_service.init_project(team_name, test_project)

    test_domain = {"actions": ["utter_1", "utter_2"],
                   "config": {"store_entities_as_slots": True},
                   "entities": [],
                   "forms": ["test_form"],
                   "intents": [{"intent_1": {"use_entities": True}},
                               {"intent_2": {"use_entities": False}}],
                   "slots": {
                       "slot_1": {"auto_fill": True,
                                  "initial_value": None,
                                  "type": "rasa_core.slots.UnfeaturizedSlot"}},
                   "templates": {"utter_1": {"text": "hello world"}}}

    projects_service.store_domain(test_domain, test_project)

    actual = projects_service.get_domain(test_project)

    assert actual == test_domain

    # Save another domain for the same project and check whether old one
    # gets replaced correctly
    test_domain["actions"].append("utter_3")
    projects_service.store_domain(test_domain, test_project)

    actual = projects_service.get_domain(test_project)

    assert actual == test_domain


def test_get_intents(projects_service: ProjectsService,
                     settings_service: SettingsService):
    test_project = "project2"
    settings_service.init_project(team_name, test_project)
    test_domain = {"intents": [{"intent_1": {"use_entities": True}},
                               {"intent_2": {"use_entities": False}}]}

    projects_service.store_domain(test_domain, test_project)
    actual = projects_service.get_intents_from_domain(test_project)

    assert actual == {"intent_1", "intent_2"}


def test_save_and_get_token(projects_service: ProjectsService):
    from uuid import uuid4
    # sql_migration successfully created token using save_token
    assert len(projects_service.get_token().token) == len(uuid4().hex)
    assert len(str(projects_service.get_token().expires)) == 10


def test_update_token(projects_service: ProjectsService):
    bot_name = 'Ursula'
    description = 'Bot that will insult you in many creative ways, be prepared!'
    projects_service.update_token(bot_name, description)
    token = projects_service.get_token()
    assert token.bot_name == bot_name
    assert token.description == description
